from EasyNN.loss.loss import Loss
from EasyNN.loss.cross_entropy.binary import Loss_BinaryCrossentropy
from EasyNN.loss.cross_entropy.categorical import Loss_CategoricalCrossentropy
from EasyNN.loss.mean_error.absolute import Loss_MeanAbsoluteError
from EasyNN.loss.mean_error.squared import Loss_MeanSquaredError